<?php

namespace Drupal\entity_usage_integrity\FormIntegrityValidation;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityWithPluginCollectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\entity_usage_integrity\IntegrityValidator;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provide integrity validation on submitted content entity edit form.
 *
 * This alter displays warnings or errors, when entity edit form is saved
 * and there are invalid relations.
 *
 * If 'block' mode is selected, saving entity with broken usage relations
 * is forbidden. If 'warning' mode is selected, saving entity with broken
 * usage relations is allowed, but warnings will be displayed.
 *
 * @see IntegritySettingsForm::buildForm()
 */
final class SubmittedEditForm extends SubmittedFormBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public function __construct(IntegrityValidator $integrity_validator, EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger, ConfigFactoryInterface $config_factory, ModuleHandlerInterface $module_handler) {
    parent::__construct($integrity_validator, $entity_type_manager, $messenger, $config_factory);
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_usage_integrity.validator'),
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('config.factory'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityWithFormValues(array &$form, FormStateInterface $form_state) {
    $form_state_copy = clone $form_state;
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $form_state->getFormObject()->getEntity();

    $form_state_copy->cleanValues();
    $values = $form_state_copy->getValues();

    if ($entity->getEntityType()->isRevisionable() && isset($values['revision']) && $values['revision']) {
      /** @var \Drupal\Core\Entity\ContentEntityStorageInterface $storage */
      $storage = \Drupal::entityTypeManager()->getStorage($entity->getEntityTypeId());
      $entity = $storage->createRevision($entity, $entity->isDefaultRevision());
    }
    else {
      $entity = clone $entity;
    }

    if ($entity instanceof EntityWithPluginCollectionInterface) {
      // Do not manually update values represented by plugin collections.
      $values = array_diff_key($values, $entity->getPluginCollections());
    }

    foreach ($values as $key => $value) {
      // TODO This foreach() is because entity_reference_override module
      //   stores data in strange format:
      //   [
      //     'target_id' => ['target_id' => xxxxxx],
      //     'overriden' => 'xxxxxxx'
      //   ]
      //   It is expected that target_id will be scalar, not array with
      //   repeated target_id key. I have no better way by now to fix that
      //   and this is temporary hotfix that has to be investigated and
      //   fixed in the future.
      if (is_array($value)) {
        foreach ($value as &$item) {
          if (isset($item['target_id']['target_id'])) {
            $item['target_id'] = reset($item['target_id']);
          }
        }
      }

      if ($entity->hasField($key)) {
        $entity->set($key, $value);
      }
    }

    // This is e.g. for content moderation.
    $this->moduleHandler->invokeAll('entity_presave', [$entity]);

    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  protected function isApplicable(FormStateInterface $form_state) {
    $form_object = $form_state->getFormObject();
    if ($form_object instanceof ContentEntityFormInterface) {
      return in_array($form_object->getOperation(), ['edit', 'default']) && !$form_object->getEntity()->isNew();
    }
    return FALSE;
  }

}
